; This is a set of sound routines that are callable from
; any large model language the uses C type calling conventions


_TEXT		SEGMENT BYTE PUBLIC 'Code'
		ASSUME	CS:_TEXT, DS:_TEXT

		public	installit		; installs the interrupt
		public	removeit		; removes the interrupt
		public	dosound			; sets the interrupt pointer to the sound data
		public	soundoff		; turns the sound off
		public	soundsareon		; inquire if sound is on or off
		public	eventsleft		; get the number of events that remain
		public	pausesound		; pause the sound
		public	unpausesound		; unpause the sound


; Returns zero if sound is of, non-zero if it's on
; soundflag = soundsareon()

soundsareon	proc	far

		push	bp
		mov	bp,sp

		push	ds		; save ds
		push	cs		; set ds
		pop	ds		; to cs

		mov	bx,sound_flag	; get sound_flag
		mov	ax,[bp+8]
		mov	ds,ax
		mov	si,[bp+6]
		mov	ds:[si],bx

		pop	ds		; restore ds
		pop	bp

		ret	4		; back again

soundsareon   	endp			; end of _sounds_are_on


; Returns the number of remaining events
; events = eventsleft();

eventsleft    	proc	far

		push	bp
		mov	bp,sp

		push	ds		; save ds
		push	cs		; set ds
		pop	ds		; to cs

		mov	bx,events	; get number of events
		mov	ax,[bp+8]
		mov	ds,ax
		mov	si,[bp+6]
		mov	ds:[si],bx

		pop	ds		; restore ds
		pop	bp

		ret	4		; back to caller

eventsleft     	endp			; end of _events_left


; Pause the current sound if there is one
; If no sound is playing, this has no effect
; pausesound()

pausesound     	proc	far

		push	ds		; save ds
		push	ax		; save ax
		push	cs		; set ds
		pop	ds		; to cs

		mov	ax,sound_flag	; get sound_flag
		cmp	ax,0		; see if there is a sound enabled
		je	no_pause_necessary	; if not, don't pause
		sub	ax,ax		; zero ax
		inc	al		; make ax=1
		mov	pause,ax	; store ax

no_pause_necessary:
		pop	ax		; restore
		pop	ds		; ax and ds

		ret			; back to caller

pausesound     	endp			; end of _pause_sound


; Pause the current sound if there is one
; If no sound is playing, this has no effect
; unpausesound()

unpausesound  	proc	far

		push	ds		; save ds
		push	ax		; save ax
		push	cs		; set ds
		pop	ds		; to cs

		sub	ax,ax		; zero ax
		mov	pause,ax	; store in pause flag

		pop	ax		; restore
		pop	ds		; ax and ds

		ret			; back to caller

unpausesound	endp			; end of _unpause_sound


; Installs the interrupt
; installit()

installit	proc	far

		jmp	past_data

old_vector	DW 0,0				; save the old 1c vector
temp		DB 0				; timer variable
events		DW 0				; number of events
sound_flag	DW 0				; flag to indicate the sound is on
data_pointer	DW 0,0				; holds the address of the sound data
pause		DW 0

past_data:
		push	ds			; save
		push	es			; some
		push	ax			; registers
		push	dx			; that we'll used

		push	cs			; set ds
		pop	ds			; to cs

		sub	ax,ax			; zero ax
		mov	events,ax		; store it in events

		mov	ax,351ch		; get the timer interrupt vector
		int	21h			; using the DOS interrupt

		mov	old_vector,bx		; store the vector offset
		mov	old_vector+2,es		; store the vector segment

		lea	dx,interrupt		; load the address of my routine
		mov	ax,251ch		; put it in the chain
		int	21h			; using the DOS interrupt

		pop	dx			; restore
		pop	ax			; the
		pop	es			; registers that
		pop	ds			; were used

		ret				; and leave

installit	endp				; end of _installit

; Removes the interrupt
; removeit()

removeit	proc	far

		push	ds			; save
		push	ax			; some
		push	dx			; registers

		push	cs			; set ds
		pop	ds			; to cs

		mov	dx,old_vector		; get the old 1c vector offset
		mov	ds,old_vector+2		; get the old 1c vector segment
		mov	ax,251ch		; reset it in the chain
		int	21h			; using the DOS interrupt

		pop	dx			; restore
		pop	ax			; the registers
		pop	ds			; we used

		ret				; leave

removeit	endp				; end of _removeit


; sound_data is in the format
; First word is number of events
; The comes three bytes per event: byte three is the time in 18ths of a second
; and bytes one and two are the frequency value
; dosound( (char far *) sound_data )

dosound       	proc	far

		push	bp			; save bp
		mov	bp,sp			; set to get stack values
		push	ds			; save the
		push	es			; registers
		push	si			; that are
		push	ax			; used

		mov	cx,ds

		push	cs			; set ds
		pop	ds			; to cs

		call	insoundoff		; first turn off the old sound

		sub	ax,ax			; zero ax
		mov	pause,ax		; reset pause flag

		mov	si,[bp+6]
		mov	es,cx
		mov	si,es:[si+2]

		mov	ax,si			; to sound data
		add	ax,2			; point past number of events word
		mov	data_pointer,ax		; set data_pointer variable
		mov	ax,es			; set es to point to sound data segment
		mov	data_pointer+2,ax	; store the segment in data_pointer

		mov	ax,es:[si]		; fetch the number of events
		mov	events,ax		; store it
		sub	al,al			; zero al
		mov	temp,al			; store al and zero temp

		pop	ax			; pop the
		pop	si			; registers
		pop	es			; that we
		pop	ds			; used and
		pop	bp			; abused

		ret	4			; out of here!

dosound   	endp				; end of _dosound


; This is the interrupt handler that actually plays the sounds
; It chains to other int 1c interrupts
; Not callable from C

interrupt	proc	far

		cli				; disable other interrupts


		; I did the next three instructions to avoid
		; having one extra word pushed on the stack.
		; Interrupts have a severe stack limit.
		push	ds			; save ds
		push	cs			; set ds
		pop	ds			; to cs

		push	es			; save the
		push	ax			; registers
		push	dx			; that we
		push	di			; use

		mov	ax,pause		; first check the pause flag
		cmp	ax,0			; see if it's zero
		je	no_pause_set		; do the interrupt sound routine if it is
		jmp	out_of_here		; otherwise get out

no_pause_set:
		mov	ax,events		; Get the number of events that remain.
		cmp	ax,0			; see if there are any.
		je	no_events		; If not, go to no_events.
						; No events means that the 
						; sound is over.

		mov	al,temp			; get the time remaining for this event
		cmp	al,0			; see if it's zero
		jne	no_events		; if not, go to no_events
						; A zero value means that the
						; duration value has elapsed and
						; we're ready for the next
						; event (if there is another).

		mov	di,data_pointer		; get the sound data offset
		mov	ax,data_pointer+2	; get the sound data segment
		mov	es,ax			; set es to the sound data segment
		mov	al,es:[di]		; get the first byte (frequency byte one)
		or	al,es:[di+1]		; or the second byte (frequency byte two)
		cmp	al,0			; see if it's zero
		je	zero_data		; if so, jmp to zero_data

		sub	ah,ah			; zero high byte
		mov	sound_flag,ax		; set sound_flag
		mov	dx,61h			; 8253 port
		in	al,dx			; get what's there		or	al,3			; set it for sound
		or	al,3			; set for sound
		out	dx,al			; send it out
		mov	dx,43h			; now set 8253 mode
		mov	al,0b6h			; for sound
		out	dx,al			; send it out
		dec	dx			; set to 8253 channel 2
		mov	al,es:[di]		; get the first byte of frequency data
		inc	di			; increment the pointer
		out	dx,al			; send it out
		mov	al,es:[di]		; get the next byte of frequency data
		inc	di			; increment the pointer
		out	dx,al			; send it out
		jmp	past_zero_data		; jmp to an entry point past the other stuff

zero_data:
		sub	ax,ax			; zero ax
		mov	sound_flag,ax		; zero sound_flag
		mov	dx,61h			; 8253 port
		in	al,dx			; get what's there
		and	al,0fch			; mask for no sound
		out	dx,al			; send it out
		add	di,2			; increment the sound data pointer

past_zero_data:
		mov	al,es:[di]		; get the duration value for the event
		inc	di			; increment the pointer
		dec	al			; subtract one to account for this go 'round
		mov	temp,al			; store it
		mov	ax,events		; get number of events
		dec	ax			; decrement value
		mov	events,ax		; store it
		mov	data_pointer,di		; save the pointer to the sound data
		jmp	out_of_here		; jmp to the section that exits

no_events:
		mov	al,temp			; no new events so
		dec	al			; decrement the timer
		mov	temp,al			; variable

out_of_here:
		pop	di			; pop
		pop	dx			; the
		pop	ax			; registers
		pop	es			; that
		pop	ds			; we used

		sti				; enable interrupts
		pushf				; push flags to simulate interrupt
		call	DWORD PTR cs:old_vector	; call the old timer routine
		iret				; iret gets us safely out of here

interrupt	endp				; end of interrupt


; Turn the sound off
; soundoff()

soundoff	proc	far

		push	ds			; save some
		push	dx			; registers
		push	ax			; that we'll use

		push	cs			; set ds
		pop	ds			; to cs

		sub	ax,ax			; zero ax
		mov	events,ax		; put it in events

		mov	dx,61h			; 8253 port
		in	al,dx			; get what's there
		and	al,0fch			; mask for no sound
		out	dx,al			; send it out

		pop	ax			; pop the
		pop	dx			; registers
		pop	ds			; we used

		ret				; and get out

soundoff	endp				; end of _soundoff


; Turn the sound off for local use

insoundoff	proc

		push	ds			; save some
		push	dx			; registers
		push	ax			; that we'll use

		push	cs			; set ds
		pop	ds			; to cs

		sub	ax,ax			; zero ax
		mov	events,ax		; put it in events

		mov	dx,61h			; 8253 port
		in	al,dx			; get what's there
		and	al,0fch			; mask for no sound
		out	dx,al			; send it out

		pop	ax			; pop the
		pop	dx			; registers
		pop	ds			; we used

		ret				; and get out

insoundoff	endp				; end of _soundoff

_TEXT		ENDS

		END



